/*
 * This software was developed at the National Institute of Standards and
 * Technology (NIST) by employees of the Federal Government in the course
 * of their official duties. Pursuant to title 17 Section 105 of the
 * United States Code, this software is not subject to copyright protection
 * and is in the public domain. NIST assumes no responsibility  whatsoever for
 * its use by other parties, and makes no guarantees, expressed or implied,
 * about its quality, reliability, or any other characteristic.
 */

/*
 * Author: George Quinn, NIST
 * Email comments to gw@nist.gov
 */

#include <stdlib.h>
#include <string.h>

#include <stdio.h>

#include "mbe.h"

#define SDK_ID		"9999"
#define SDK_EMAIL	"gw@nist.gov"

#define TEMPLATE_DATA	"'Twas brillig, and the sligthy toves\n" \
                        "Did gyre and gimble in the wabe.\n"

/* Minus one to ignore null terminator */
#define TEMPLATE_SIZE	( sizeof(TEMPLATE_DATA) - 1 )

double random_score()
{
   return rand() / (double) RAND_MAX;
}


int32_t get_pid(
   char* SDK_IDentifier,
   char* email_address)
{
   memcpy(SDK_IDentifier, SDK_ID, 5);
   memcpy(email_address, SDK_EMAIL, 12);

   return 0;
}

int32_t get_max_template_sizes(
   uint32_t* max_enrollment_template_size,
   uint32_t* max_recognition_template_size)
{
   *max_enrollment_template_size =
      *max_recognition_template_size = TEMPLATE_SIZE;

   return 0;
}

#ifdef CLASS_D

int32_t initialize_frontal_pose_estimation(
   const char* configuration_location)
{
   return 0;
}

int32_t estimate_frontal_pose_conformance(
   const MULTIFACE* input_faces,
   double* non_frontalities)
{
   return 0;
}

#endif

#ifdef MBE_CLASS_A

int32_t initialize_verification(
   const char* configuration_location,
   const char** descriptions,
   const uint8_t num_descriptions)
{
   return 0;
}

int32_t convert_multiface_to_enrollment_template(
   const MULTIFACE* input_faces,
   uint32_t* template_size,
   uint8_t* proprietary_template)
{
   *template_size = TEMPLATE_SIZE;
   memcpy(proprietary_template, TEMPLATE_DATA, TEMPLATE_SIZE);

   return 0;
}

int32_t convert_multiface_to_verification_template(
   const MULTIFACE* input_faces,
   uint32_t* template_size,
   uint8_t* proprietary_template,
   uint8_t* quality)
{
   *template_size = TEMPLATE_SIZE;
   memcpy(proprietary_template, TEMPLATE_DATA, TEMPLATE_SIZE);

   *quality = 254;

   return 0;
}

int32_t match_templates(
   const uint8_t* verification_template,
   const uint32_t verification_template_size,
   const uint8_t* enrollment_template,
   const uint32_t enrollment_template_size,
   double* similarity)
{
   *similarity = random_score();

   return 0;
}

#endif

#if defined(MBE_CLASS_B) || defined(MBE_CLASS_C)

int32_t initialize_enrollment_session(
   const char* configuration_location,
   const char* enrollment_directory,
   const uint32_t num_persons,
   const uint32_t num_images,
   const char** descriptions,
   const uint8_t num_descriptions)
{
   return 0;
}

int32_t convert_multiface_to_enrollment_template(
   const MULTIFACE* input_faces,
   EYEPAIR* const* output_eyes,
   uint32_t *template_size,
   uint8_t* proprietary_template)
{
   unsigned int i;

   for (i = 0; i < input_faces->numfaces; ++i)
   {
      EYEPAIR* e = output_eyes[i];

      e->failed = 0;
      e->xleft = e->xright = e->yleft = e->yright = 0;
   }

   *template_size = TEMPLATE_SIZE;
   memcpy(proprietary_template, TEMPLATE_DATA, TEMPLATE_SIZE);

   return 0;
}

int32_t finalize_enrollment(
   const char* enrollment_directory,
   const char* edb_name,
   const char* edb_manifest_name)
{
   return 0;
}


int32_t initialize_identification_session(
   const char* config_directory,
   const char* enrollment_directory)
{
   return 0;
}

int32_t initialize_feature_extraction_session(
   const char* configuration_location,
   const char* enrollment_directory)
{
   return 0;
}

int32_t convert_multiface_to_identification_template(
   const MULTIFACE* input_faces,
   EYEPAIR* const* output_eyes,
   uint32_t* template_size,
   uint8_t* identification_template)
{
   unsigned int i;

   for (i = 0; i < input_faces->numfaces; ++i)
   {
      EYEPAIR* e = output_eyes[i];

      e->failed = 0;
      e->xleft = e->xright = e->yleft = e->yright = 0;
   }

   *template_size = TEMPLATE_SIZE;
   memcpy(identification_template, TEMPLATE_DATA, TEMPLATE_SIZE);

   return 0;
}

#endif

#ifdef MBE_CLASS_C

int32_t identify_template(
   const uint8_t* identification_template,
   const uint32_t identification_template_size,
   const uint32_t candidate_list_length,
   CANDIDATE* const * candidate_list)
{
   uint32_t i;
   for (i = 0; i < candidate_list_length; i++)
   {
      candidate_list[i]->failed = 0;
      candidate_list[i]->template_id = i;
      candidate_list[i]->similarity_score = random_score();
      candidate_list[i]->probability      = random_score();
   }

   return 0;
}

#endif

#ifdef MBE_CLASS_B

int32_t verify_template(const uint8_t* verification_template,
   const uint32_t verification_template_size,
   const uint32_t enrolled_identity_claim,
   double* similarity)
{
   *similarity = random_score();

   return 0;
}

#endif

