/*
 * This software was developed at the National Institute of Standards and
 * Technology (NIST) by employees of the Federal Government in the course
 * of their official duties. Pursuant to title 17 Section 105 of the
 * United States Code, this software is not subject to copyright protection
 * and is in the public domain. NIST assumes no responsibility  whatsoever for
 * its use by other parties, and makes no guarantees, expressed or implied,
 * about its quality, reliability, or any other characteristic.
 */

/*
 * Author: George Quinn, NIST
 * Email comments to gw@nist.gov
 */

#include "mbe_validation.h"

#include <iostream>
#include <iomanip>	// for setprecision
#include <cstring>	// for memcpy, size_t
#include <cstdarg>	// for va_list

extern "C" {
   #include "jpeglib.h"
}

ostream& operator<<(ostream& os, const EYEPAIR& e)
{
   os << (int)e.failed << " (" << e.xleft << "," << e.yleft << ") ("
      << e.xright << "," << e.yright << ")" << endl;

   return os;
}

ostream& operator<<(ostream& os,  const CANDIDATE& c)
{
   os << fixed << setprecision(score_precision)
      << (int)c.failed << " " << c.template_id << " " << c.similarity_score << " "
      << c.probability;

   return os;
}

istream& operator>>(istream& is, ONEFACE& oneface)
{
   if (is)
   {
      string img_path,
             description;
  
      is >> description  >> oneface.dob    >> oneface.mob    >> oneface.yob  
         >> oneface.day  >> oneface.month  >> oneface.year   >> oneface.sex
         >> oneface.race >> oneface.weight >> oneface.height >> img_path;

      if ( description == "mugshot" )
      {
         oneface.description = (char*)"mugshot"; // string not actually writable.

      } else
      {
         fail("Unsupported image description %s.", description.c_str());
      }

      // If has ppm extension.
      if ( img_path.rfind(".ppm") == img_path.length() - 4 )
      {
         oneface.format = 01; // originally was jpeg

         ifstream image_stream(img_path.c_str(), ios::binary);
         if (! image_stream )
            fail("Could not read image %s.", img_path.c_str());

         string magic_number;
         int max_color;

         image_stream >> magic_number >> oneface.image_width >> oneface.image_height
                      >> max_color;

         image_stream.ignore();	// disregard trailing whitespace

         if (magic_number != "P6")
            fail("Magic number test failed for %s.", img_path.c_str());

         oneface.image_depth = 24;

         const uint32_t image_size = oneface.image_width * oneface.image_height *
                                     oneface.image_depth / 8;

         oneface.data = new uint8_t[ image_size ];

         image_stream.read( (char*)oneface.data, image_size);

         image_stream.close();

         /* 
            ofstream os("test.ppm", ios::binary);
            if (!os)
               fail("Failed debug test.");

            os << "P6" << endl << oneface.image_width << " " << oneface.image_height
               << endl << max_color << endl;

            os.write( (char*)oneface.data, image_size);
            os.close();
         */
         

      // if not ppm
      } else {
         fail("Invalid image format for %s.", img_path.c_str());
      }

   } // end if valid stream

   return is;
}

void fail(const char* format, ...)
{
   va_list args;
   va_start(args, format);
   vfprintf(stderr, format, args);
   va_end(args);

   fprintf(stderr, "\nTerminating.\n");
   exit(EXIT_FAILURE);
}

