/*
 * This software was developed at the National Institute of Standards and
 * Technology (NIST) by employees of the Federal Government in the course
 * of their official duties. Pursuant to title 17 Section 105 of the
 * United States Code, this software is not subject to copyright protection
 * and is in the public domain. NIST assumes no responsibility  whatsoever for
 * its use by other parties, and makes no guarantees, expressed or implied,
 * about its quality, reliability, or any other characteristic.
 */

/*
 * Author: George Quinn, NIST
 * Email comments to gw@nist.gov
 */

#include "person.h"
#include "mbe_validation.h"

#include <iostream>

using namespace std;

person::person() : pid_(0)
{
   mface_.numfaces = 0;
   mface_.faces = new ONEFACE*[max_num_faces];
}

person::~person()
{
   clear();
  
   delete [] mface_.faces;
}

void person::add_face(const ONEFACE& oneface)
{
   mface_.faces[ mface_.numfaces++ ] = new ONEFACE(oneface);
}

void person::clear()
{
   while (mface_.numfaces)
   {
      delete [] mface_.faces[ --mface_.numfaces ]->data;
      delete mface_.faces[   mface_.numfaces ];
   }
 
   pid_ = 0;
}

person& person::operator=(const person& p)
{

   pid_ = p.pid_;

   mface_.numfaces = p.mface_.numfaces;
   mface_.faces = new ONEFACE*[max_num_faces];

   for (uint32_t i = 0; i < mface_.numfaces; ++i)
   {
      mface_.faces[i] = new ONEFACE;

      ONEFACE& oneface = *mface_.faces[i];
      oneface = *p.mface_.faces[i];

      const uint32_t size = oneface.image_width * oneface.image_height * oneface.image_depth;

      oneface.data = new uint8_t[size];

      memcpy(oneface.data, mface_.faces[i]->data, size);
   }

   return *this;

}


istream& operator>>(istream& is, person& p)
{
   p.clear();

   if (is >> p.pid_)
   {      
      uint32_t curr_pid = p.get_pid();
      streampos pos;

      do
      {
         ONEFACE oneface;

         is >> oneface;
         p.add_face(oneface);

         pos = is.tellg();

      } while ( is >> curr_pid && curr_pid == p.get_pid() );

      is.seekg(pos);     // unread last pid
      is.clear();        // reset EOF flag
   }

   return is;
}




